#include <stdio.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

typedef enum {
	MOTOR_A,
	MOTOR_B,
} Motor;


typedef enum {
	FLOAT,
	FORWARD,
	BACKWARD,
	BREAK,
} Direction;

const int ADC_channel[] = {0, 1, 2, 3, 4, 5, 6, 7};

const int num_channels = 8;

#define ADC_INIT_CHANNEL 0

int ADC_value[num_channels];

void ADC_init(void)
{	
	ADMUX = 0;
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);

}

int ADC_get(uint8_t channel)
{
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));

	return ADC;
}

void initPWM(void)
{
	/* setez ca output pinul PB3 --> OC0A */
	DDRB |= (1 << PB3); 
	/* setez ca output pinul PB3 --> OC0B */
	DDRB |= (1 << PB4); 
		
	/* set fast pwm 8-bit, top = 0xFF */
	/* setez modul non-inverting */
	TCCR0A |= (1 << WGM01) | (1 << WGM00) | (1 << COM0A1) | (1 << COM0B1);

}
void setMotorDirection(Motor motor, Direction dir) {
	if(motor == MOTOR_A) {
		/* folosim PORTC -- AIN1 = PC1; AIN2 = PC2 */
		if(dir == FLOAT) {
			PORTC = (0 << PC1) | (0 << PC2);
		} else if (dir == FORWARD) {
			PORTC = (0 << PC1) | (1 << PC2);
		} else if (dir == BACKWARD) {
			PORTC = (1 << PC1) | (0 << PC2);
		} else {
			//BREAK
			PORTC = (1 << PC1) | (1 << PC2);
		}
	} else {
		/* folosim PORTC -- BIN1 = PC3; BIN2 = PC4 */
		if(dir == FLOAT) {
			PORTC = (0 << PC3) | (0 << PC4);
		} else if (dir == FORWARD) {
			PORTC = (0 << PC3) | (1 << PC4);
		} else if (dir == BACKWARD) {
			PORTC = (1 << PC3) | (0 << PC4);
		} else {
			//BREAK
			PORTC = (1 << PC3) | (1 << PC4);
		}
	}
}
void initGPIO(void) {
	

	/*
	 * Setari pentru senzori
	 */
	/* Setez pinii PA0-PA7 - ca intrare. */
	int i;
	for(i = 0; i < 8; i++) {
		DDRA &= ~(1 << i);
	}
	/* Activez rezistenta de pull-up pentru pinii PA0-PA5. */
	for(i = 0; i < 8; i++) {
		PORTA |= (1 << i);
	}
	/*
	 * Setari driver motoare
	 */
	
	/* Setez pinii PC-PC3 - ca iesire. */
	DDRC |= (1 << PC0); //AIN1
	DDRC |= (1 << PC1); //AIN2
	DDRC |= (1 << PC2); //BIN1
	DDRC |= (1 << PC3); //BIN2
	
	/* Setez pinii PC5-PC8 ca intrare */
	DDRC &= ~(1 << PC4);//OUTB-M2
	PORTC |= (1 << PC4);
	
	DDRC &= ~(1 << PC5);//OUTA-M2
	PORTC |= (1 << PC5);
	
	DDRC &= ~(1 << PC6);//OUTB-M1
	PORTC |= (1 << PC6);
	
	DDRC &= ~(1 << PC7);//OUTA-M1
	PORTC |= (1 << PC7);
	
	
	/* Setez LED-ul */
	DDRD &= ~(1 << PD7);//OUTA-M1
	PORTD |= (1 << PD7);
	
	/* Pun fiecare motor in starea float */
	setMotorDirection(MOTOR_A, FLOAT);
	setMotorDirection(MOTOR_B, FLOAT); 
}

void setMotorPwm(Motor motor, unsigned int speed){
	if(motor == MOTOR_A) {
		OCR0A = speed;
	} else {
		OCR2A = speed;
	}
}

int main(void) {
	
	initGPIO();
	initPWM();
	
	/* Duty cicle 0 pt ambele - initial */
	OCR0A = 0;
	OCR0B = 0;
	
	ADC_init();
	
	/* Pun fiecare motor in starea forward */
	setMotorDirection(MOTOR_A, FORWARD);
	setMotorDirection(MOTOR_B, FORWARD); 
	
	_delay_ms(3000);
	while(1){	
		if(ADC_get(0) > 340 && ADC_get(1)  > 340 ) { // //negru
			/* Aprinde LED */
			PORTD |= (1 << PD7);
			
			/* Mergi spre stanga */
			OCR0A = 0xFF/4;
			OCR0B = 0;
		} else if (ADC_get(6) > 340 && ADC_get(7) > 340){
			/* Stinge led */
			PORTD &= ~(1 <<  PD7);
			/* Mergi spre dreapta */
			OCR0A = 0;
			OCR0B = 0xFF/4;
		} else {
			/* Mergi inainte */
			OCR0A = 0xFF/6;
			OCR0B = 0xFF/6;	
		}
	}
	return 0;
}



